@@ -116,9 +116,13 @@ def activate_line(x, y, z, command, tile_type=""):
 		switch2 = command.split(None, 2)[1].lower()
 		#interpret the second part of the command
 		set_to = command.split(None, 2)[2]
-		set_to = activate_line(x, y, z, command.split(None, 2)[2])
+		if switch2 != "name":
+			set_to = activate_line(x, y, z, command.split(None, 2)[2])
 
-		if switch2 == "hp":
+		if switch2 == "name":
+			#note that name is not interpreted as a command. See above.
+			g.name = set_to
+		elif switch2 == "hp":
 			g.hp = g.hp + int(set_to)
 			if g.hp >= g.adj_maxhp: g.hp = g.adj_maxhp
 		elif switch2 == "ep":
@@ -341,7 +345,9 @@ def activate_line(x, y, z, command, tile_type=""):
 
 	elif switch == "stat":  #return stat
 		switch2 = command.split(None, 2)[1].lower()
-		if switch2 == "hp":
+		if switch2 == "name":
+			return_num = g.name
+		elif switch2 == "hp":
 			return_num = g.hp
 		elif switch2 == "ep":
 			return_num = g.ep
