@@ -22,7 +22,7 @@ hp = StringVar()
 ep = StringVar()
 
 help_text = StringVar()
-return_from_yesno = StringVar()
+return_from_dialog = StringVar()
 
 #location of the various buttons. (Pixels from the left of the button canvas)
 attack_button_loc = 0
@@ -54,6 +54,7 @@ base_mon_hp_start = []
 base_mon_hp_width = []
 #y location of the monster hp bar per monster. Used to redraw it.
 base_mon_hp_y_start = []
+base_mon_hp_height = []
 
 #refreshes the battle view. Call after changing anything.
 def refresh():
@@ -89,20 +90,138 @@ def refresh():
 
 
 #attack button was pressed
-#FIX for multimonster
 def attack(event=0):
 	global cur_button
-	if monster_list[cur_monster].hp > 0 and g.hp > 0:
-		attack_monster(cur_monster)
-	for i in range(len(monster_list)):
-		if monster_list[i].hp > 0 and g.hp > 0:
-			attack_player(i)
-	if monster_list[cur_monster].hp > 0:
+	cur_button = 0
+	if can_leave() == 1: return 0
+	last_mon_num = select_monster()
+	if last_mon_num == -1: return 0
+
+	#Both hp's *should* be over 0. Just a precaution.
+	if monster_list[last_mon_num].hp > 0 and g.hp > 0:
+		attack_monster(last_mon_num)
+	attack_player()
+	if can_leave() == 0:
 		refresh()
 		refresh_buttons()
 
+
+#Returns the array location in monster_list of the monster the player wants to
+#attack, or -1 on cancel. If more than one monster exists, displays a selection
+#arrow.
+def select_monster():
+	#find out if we need to make the player select a monster.
+	num_of_monsters = 0
+	last_mon_num = 0
+	for i in range(len(monster_list)):
+		if monster_list[i].hp > 0:
+			num_of_monsters += 1
+			last_mon_num = i
+
+	#If there is only one living monster, last_mon_num holds the array location
+	#in monster_list[]. Otherwise, ask the player which monster to attack.
+	if num_of_monsters == 1: return last_mon_num
+	action.has_dialog = 1
+	global active_button
+	for i in range(len(monster_list)):
+		if monster_list[i].hp > 0:
+			active_button = i
+			break
+	else:
+		print "BUG: select_monster called when all monsters were dead"
+		return -1
+
+	global return_from_dialog
+	return_from_dialog.set("0")
+
+	canvas_mon_pic.create_image(
+		base_mon_hp_start[active_button],
+		base_mon_hp_y_start[active_button]-20,
+		anchor=N, image=monster_select_arrow, tags="monster_arrow")
+	#bindings
+	bind_attack_keys()
+
+	#wait. Continue after activate_yesno() is run.
+	window_battle.wait_variable(return_from_dialog)
+	#cleanup
+	if return_from_dialog.get() == "2":
+		return -1
+	else:
+		try: canvas_mon_pic.delete("monster_arrow")
+		except TclError: pass
+		bind_keys()
+
+		action.has_dialog = 0
+		return active_button
+
+#Cycle through the various monsters when attacking.
+def choose_monster_prev(event=None):
+	global active_button
+	i = active_button
+	while 1==1:
+		i -= 1
+		if i == -1:
+			i = len(monster_list) - 1
+		if monster_list[i].hp > 0:
+			active_button = i
+			break
+		if i == active_button:
+			print "choose_monster_prev couldn't find a monster."
+			return 0
+	canvas_mon_pic.coords(canvas_mon_pic.find_withtag("monster_arrow"),
+				base_mon_hp_start[i],
+				base_mon_hp_y_start[i]-20)
+	help_text.set("Attack the " + monster_list[active_button].name)
+
+def choose_monster_next(event=None):
+	global active_button
+	i = active_button
+	while 1==1:
+		i += 1
+		if i == len(monster_list):
+			i = 0
+		if monster_list[i].hp > 0:
+			active_button = i
+			break
+		if i == active_button:
+			print "choose_monster_next couldn't find a monster."
+			return 0
+	canvas_mon_pic.coords(canvas_mon_pic.find_withtag("monster_arrow"),
+				base_mon_hp_start[i],
+				base_mon_hp_y_start[i]-20)
+	help_text.set("Attack the " + monster_list[active_button].name)
+
+#called on Return when the yesno dialog box is active.
+def choose_monster(event=None):
+	global return_from_dialog
+	return_from_dialog.set("1")
+
+#Cancels the monster selection. Called on Esc.
+def cancel_monster(event=None):
+	global return_from_dialog
+	return_from_dialog.set("2")
+
+
+#Called whenever the mouse moves in the monster canvas. Used to select the
+#proper monster, given the x and y location of the mouse.
+def monster_mouse_move(event=None):
+	global active_button
+	for i in range(len(monster_list)):
+		if monster_list[i].hp > 0:
+			if event.x >= base_mon_hp_start[i] and \
+			event.x <= base_mon_hp_start[i] + base_mon_hp_width[i] and \
+			event.y >= base_mon_hp_y_start[i] - base_mon_hp_height[i] and \
+			event.y <= base_mon_hp_y_start[i]:
+				active_button = i
+				canvas_mon_pic.coords(
+						canvas_mon_pic.find_withtag("monster_arrow"),
+						base_mon_hp_start[i],
+						base_mon_hp_y_start[i]-20)
+				help_text.set("Attack the " + monster_list[i].name)
+
 #attack the i'th monster in monster_list[].
 def attack_monster(i):
+	if i == -1: return
 	global cur_mon_hp
 	#find the damage done
 	damage = g.die_roll(1, g.adj_attack + 2)
@@ -119,9 +238,15 @@ def attack_monster(i):
 		main.print_message("You miss the " +
 				monster_list[i].name + ".")
 
+#Make all monsters attack the player.
+def attack_player():
+	for i in range(len(monster_list)):
+		if monster_list[i].hp > 0 and g.hp > 0:
+			attack_player_per_monster(i)
 
-#monster attacks you
-def attack_player(i):
+
+#Make the i'th monster attack you
+def attack_player_per_monster(i):
 	damage = g.die_roll(1, monster_list[i].attack + 2)
 	damage = damage - g.die_roll(1, g.adj_defense + 2)
 	if damage > 0:
@@ -134,6 +259,8 @@ def attack_player(i):
 			main.print_message("The " + monster_list[i].name +
 			" kills you.")
 			cur_button = 1
+			refresh()
+			refresh_buttons()
 
 	elif monster_list[i].hp > 0:
 		main.print_message("The " + monster_list[i].name +
@@ -141,7 +268,6 @@ def attack_player(i):
 
 
 #use to reward the player for killing a monster
-#FIX for multimonster
 def monster_dead(i):
 	#make the display look better
 	monster_list[i].hp = 0
@@ -158,28 +284,40 @@ def monster_dead(i):
 		main.print_message("and get " + str(exp) + " experience.")
 		g.add_exp(exp)
 	else:
-		i=0
-		while i < len(monster_list[i].on_death):
+		j=0
+		while j < len(monster_list[i].on_death):
 			temp = action.activate_line(g.xgrid, g.ygrid, g.zgrid,
-				monster_list[i].on_death[i])
+				monster_list[i].on_death[j])
 			if temp == "end": break
 			if main.dead_yet() == 1:
 				break
-			i += 1
+			j += 1
 
 	#finish up
-	window_battle.destroy()
-	g.cur_window = "main"
-	global did_run
-	did_run = 0
+	if can_leave() == 1:
+		window_battle.destroy()
+		g.cur_window = "main"
+		global did_run
+		did_run = 0
+	else:
+		refresh()
+
+#returns 1 if battle is finished, (one of the parties is dead) or 0 if not.
+def can_leave():
+	if g.hp <= 0:
+		return 1
+	for i in range(len(monster_list)):
+		if monster_list[i].hp > 0:
+			return 0
+	return 1
+
 
 #run/leave/quit button was pressed
-#FIX for multimonster
 def runaway():
 	global run_attempts
 	try:
 		#leave/quit
-		if g.hp <= 0 or monster_list[cur_monster].hp <= 0:
+		if can_leave() == 1:
 			window_battle.destroy()
 		#Attempt to run. Testers get better chance.
 		else:
@@ -187,21 +325,21 @@ def runaway():
 					(g.name == "testing123" and g.die_roll(1, 10) > 5):
 				window_battle.destroy()
 				main.print_message("You coward! You ran away from the "
-					+ monster_list[cur_monster].name + ".")
+					+ monster.monster_groups[mon_index].name + ".")
 				global did_run
 				did_run = 1
 			else:
 				run_attempts += 1
 				main.print_message("You fail to run away.")
-				attack_player(cur_monster)
+				attack_player()
+				refresh()
 	except TclError:
 		pass
 
 #Use item button was pressed
 def useitem():
-	#don't do anything if anyone is dead.
-#FIX for multimonster
-	if monster_list[cur_monster].hp < 1 or g.hp < 1:
+	#don't do anything if battle is over.
+	if can_leave() == 1:
 		return 0
 
 	#put the name of the item into item_name
@@ -226,26 +364,26 @@ def useitem():
 					str(g.item.item[item_value].quality) + " health.")
 		g.item.drop_inv_item(g.item.find_inv_item(item_value))
 		if g.hp >= g.adj_maxhp: g.hp = g.adj_maxhp
-#FIX for multimonster
-		attack_player(cur_monster)
+		attack_player()
 
 	#if item is explosive
-#FIX for multimonster
 	if item_type == 12:
+		sel_mon = select_monster()
+		if sel_mon == -1: return 0
 		#if monster is still alive
-		if monster_list[cur_monster].hp > 0:
+		if monster_list[sel_mon].hp > 0:
 			damage = int(g.item.item[item_value].quality)
 			g.item.drop_inv_item(g.item.find_inv_item(item_value))
 			if damage > 0:
-				main.print_message("The " + monster_list[cur_monster].name +
+				main.print_message("The " + monster_list[sel_mon].name +
 					" is hit for " + str(damage) + " damage.")
-			monster_list[cur_monster].hp = monster_list[cur_monster].hp - damage
+			monster_list[sel_mon].hp = monster_list[sel_mon].hp - damage
 			#you kill it?
-			if monster_list[cur_monster].hp <= 0:
-				monster_dead(cur_monster)
-				return 0
-			else:
-				attack_player(cur_monster)
+			if monster_list[sel_mon].hp <= 0:
+				monster_dead(sel_mon)
+			if can_leave() == 0:
+				attack_player()
+	if can_leave() == 1: return 0
 	refresh()
 
 	#replace the cursor
@@ -262,7 +400,7 @@ def useitem():
 #FIX for multimonster
 def useskill():
 	#don't do anything if monster is dead.
-	if monster_list[cur_monster].hp < 1 or g.hp < 1:
+	if can_leave() == 1:
 		return 0
 
 	#get the skill name.
@@ -291,13 +429,13 @@ def useskill():
 		#increase attack ability
 		g.adj_attack += (1 + int(g.level)/4)
 		main.attack.set(g.adj_attack)
-		attack_player(cur_monster)
+		attack_player()
 
 	elif g.skill[skill_index][1] == 1: #Sneak away
 		global run_attempts
 		if g.die_roll(1, 10 + int(g.level) + run_attempts) > 4:
 			main.print_message("You coward! You ran away from the " +
-				monster_list[cur_monster].name + ".")
+				monster.monster_groups[mon_index].name + ".")
 			window_battle.destroy()
 			global did_run
 			did_run = 1
@@ -305,27 +443,30 @@ def useskill():
 		else:
 			run_attempts += 3
 			main.print_message("You fail to sneak away.")
-			attack_player(cur_monster)
+			attack_player()
 	elif g.skill[skill_index][1] == 2: #Frenzy
-		for i in range(2 + int(g.level)/3):
-			attack_monster(cur_monster)
-			if monster_list[cur_monster].hp <= 0:
+		for i in range(3 + int(g.level)/3):
+			mon_num = select_monster()
+			if mon_num == -1: return 0
+			attack_monster(mon_num)
+			if can_leave() == 1:
 #				monster_dead()
 				return 0
-		if monster_list[cur_monster].hp > 0:
-			attack_player(cur_monster)
+			refresh()
+		if can_leave() == 0: attack_player()
 	elif g.skill[skill_index][1] == 3: #Dismember
-		damage = g.adj_attack * 2
-		main.print_message("You hit the " + monster_list[cur_monster].name +
+		mon_num = select_monster()
+		if mon_num == -1: return 0
+		damage = (g.adj_attack * 5)/2
+		main.print_message("You hit the " + monster_list[mon_num].name +
 			" for " + str(damage) + " damage.")
-		monster_list[cur_monster].hp = monster_list[cur_monster].hp - damage
+		monster_list[mon_num].hp = monster_list[mon_num].hp - damage
 		#you kill it?
-		if monster_list[cur_monster].hp <= 0:
-			monster_dead(cur_monster)
-			return 0
-		else:
-			attack_player(cur_monster)
+		if monster_list[mon_num].hp <= 0:
+			monster_dead(mon_num)
+		attack_player()
 
+	if can_leave() == 1: return 0
 	refresh()
 
 	#replace the cursor
@@ -354,7 +495,7 @@ def refresh_buttons():
 			canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 				image=quit_button_sel)
 			help_text.set("Quit")
-		elif monster_list[cur_monster].hp > 0:
+		elif can_leave() == 0:
 			canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 				image=run_button_sel)
 			help_text.set("Run away")
@@ -364,7 +505,7 @@ def refresh_buttons():
 			help_text.set("Leave")
 	elif g.hp <= 0: canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 			image=quit_button)
-	elif monster_list[cur_monster].hp > 0:
+	elif can_leave == 0:
 		canvas_buttons.create_image(run_button_loc,
 			0, anchor=NW, image=run_button)
 	else: canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
@@ -562,8 +703,8 @@ def battle_show_yesno(line=""):
 	action.has_dialog = 1
 	global active_button
 	active_button = 0
-	global return_from_yesno
-	return_from_yesno.set("0")
+	global return_from_dialog
+	return_from_dialog.set("0")
 
 	canvas_width = no_button.width()*4/3+yes_button.width()*4/3
 	canvas_height = yes_button.height()+50
@@ -600,7 +741,7 @@ def battle_show_yesno(line=""):
 	listbox_skill.bind("<ButtonRelease-1>", do_nothing)
 
 	#wait. Continue after activate_yesno() is run.
-	window_battle.wait_variable(return_from_yesno)
+	window_battle.wait_variable(return_from_dialog)
 	#cleanup
 	try: canvas_battle2.destroy()
 	except TclError: return 1
@@ -632,8 +773,8 @@ def change_yesno_sel(event=None):
 
 #called on Return when the yesno dialog box is active.
 def activate_yesno(event=None):
-	global return_from_yesno
-	return_from_yesno.set("1")
+	global return_from_dialog
+	return_from_dialog.set("1")
 
 
 #start a battle.
@@ -674,6 +815,10 @@ def begin(mon_index_input):
 	no_button_sel = ImageTk.PhotoImage(file=g.mod_directory + \
                 "/images/buttons/no_sel.png")
 
+	global monster_select_arrow
+	monster_select_arrow = ImageTk.PhotoImage(file=g.mod_directory + \
+		"/images/buttons/sword_pointer.png")
+
 	# Set the current window
 	g.cur_window = "battle"
 
@@ -698,8 +843,11 @@ def begin(mon_index_input):
 	canvas_mon_pic.create_image(0, 0, anchor=NW, image=background_pic)
 	monster_pic = []
 	global base_mon_hp_start
+	base_mon_hp_start = []
 	global base_mon_hp_width
+	base_mon_hp_width = []
 	global base_mon_hp_y_start
+	base_mon_hp_y_start = []
 	for i in range(len(monster_list)):
 		if os.path.isfile(g.mod_directory +
 				"/images/tiles/monsters/" + monster_list[i].name + ".png"):
@@ -709,12 +857,20 @@ def begin(mon_index_input):
 			monster_pic.append(ImageTk.PhotoImage(file=g.mod_directory +
 				"/images/tiles/monsters/generic.png"))
 #FIX
-		canvas_mon_pic.create_image(background_pic.width()/2+i*40, 10,
+		if len(monster.monster_groups[mon_index].x_pos) > i:
+			xstart = monster.monster_groups[mon_index].x_pos[i]
+		else: xstart = background_pic.width()/2+i*40
+		if len(monster.monster_groups[mon_index].y_pos) > i:
+			ystart = monster.monster_groups[mon_index].y_pos[i]
+		else: ystart = 10
+
+		canvas_mon_pic.create_image(xstart, ystart,
 			anchor=N, image=monster_pic[i])
 
-		base_mon_hp_start.append(background_pic.width()/2 - monster_pic[i].width()/2+i*40)
+		base_mon_hp_start.append(xstart - monster_pic[i].width()/2)
 		base_mon_hp_width.append(monster_pic[i].width())
-		base_mon_hp_y_start.append(15+monster_pic[i].height())
+		base_mon_hp_height.append(monster_pic[i].height())
+		base_mon_hp_y_start.append(ystart + 5+monster_pic[i].height())
 		#Under (red) part of the monster hp display
 		canvas_mon_pic.create_rectangle(base_mon_hp_start[i], base_mon_hp_y_start[i],
 				base_mon_hp_start[i]+base_mon_hp_width[i],
@@ -724,7 +880,7 @@ def begin(mon_index_input):
 				base_mon_hp_start[i]+base_mon_hp_width[i],
 				base_mon_hp_y_start[i]+5, fill="#05BB05",
 				tags="monster_hp_rect"+ str(i))
-		canvas_mon_pic.create_text(background_pic.width()/2+i*40,
+		canvas_mon_pic.create_text(xstart,
 			base_mon_hp_y_start[i]+7, anchor=N, text="", tags="mon_hp"+str(i))
 
 	if os.path.isfile(g.mod_directory + "/images/tiles/people/hero_n.png"):
@@ -743,10 +899,6 @@ def begin(mon_index_input):
 			column=1, columnspan=2)
 
 
-	#stat display
-#	label_mon_hp = Label(pic_frame, textvariable=mon_hp,
-#		bg="#327a22").grid(row=5, column=2, sticky=E, padx=2)
-
 #FIX for multimonster
 	label_attack = Label(window_battle, text="Attack: " +
 	str(monster_list[cur_monster].attack), bg=bgcolour).grid(row=4,
@@ -852,8 +1004,7 @@ def begin(mon_index_input):
 	#bindings
 	bind_keys()
 
-	main.print_message("The " + monster.monster_groups[mon_index].name +
-		" attacks you.")
+	main.print_message(monster.monster_groups[mon_index].attack_message)
 
 	refresh()
 	refresh_buttons()
@@ -881,5 +1032,24 @@ def bind_keys():
 	listbox_item.bind("<ButtonRelease-1>", mouse_handler_item)
 	listbox_skill.bind("<ButtonRelease-1>", mouse_handler_skill)
 
+#keybindings when choosing a monster to attack.
+def bind_attack_keys():
+	main.window_main.bind(g.binding_attack, choose_monster)
+	main.window_main.bind(g.binding_cancel, cancel_monster)
+	main.window_main.bind(g.binding_action, choose_monster)
+	main.window_main.bind(g.binding_up, choose_monster_prev)
+	main.window_main.bind(g.binding_down, choose_monster_next)
+	main.window_main.bind(g.binding_left, choose_monster_prev)
+	main.window_main.bind(g.binding_right, choose_monster_next)
+	main.window_main.bind(g.binding_inv, key_handler)
+	main.window_main.bind(g.binding_save, key_handler)
+	main.window_main.bind(g.binding_quit, key_handler)
+
+	canvas_buttons.bind("<ButtonRelease-1>", do_nothing)
+	canvas_buttons.bind("<Motion>", do_nothing)
+	listbox_item.bind("<ButtonRelease-1>", do_nothing)
+	listbox_skill.bind("<ButtonRelease-1>", do_nothing)
+	canvas_mon_pic.bind("<Motion>", monster_mouse_move)
+	canvas_mon_pic.bind("<ButtonRelease-1>", choose_monster)
 def do_nothing(event=0):
 	pass
