@@ -111,6 +111,10 @@ def drop_item():
 	try: item_to_delete = item.find_inv_item(item.inv[curr_item])
 	except IndexError: return 0
 
+	if item.item[item.inv[item_to_delete]].price == 0:
+		main.print_message("You feel attached to your " +
+			item.item[item.inv[item_to_delete]].name)
+		return 0
 	#the inv[] location of the item is now in item_to_delete.
 	#Ask if the player really wants to drop it.
 
