@@ -187,8 +187,9 @@ def process_onload():
 				if dead_yet() == 1: break
 
 #recalculate and display stats. leave_att_def_alone is for calling from a
-#battle.
+#battle, with the possibility of temp bonuses to attack/defense.
 def recalc_stats(leave_att_def_alone = 0):
+	name.set(g.name)
 	if leave_att_def_alone == 0:
 		#adjust attack and defense based on equipment.
 		g.adj_attack = int(g.attack)
@@ -522,6 +523,7 @@ def activate_yesno(event=None):
 
 def cleanup(event=None):
 	activate_yesno()
+	battle.return_from_dialog.set("2")
 	window_main.destroy()
 
 #call to create window_main.
