@@ -152,6 +152,8 @@ def set_mode(mode):
 
 #set all interior tiles to background tile
 def set_bkg(bkg):
+	global background
+	background = bkg
 	for y in range(mapsize_y-2):
 		for x in range(mapsize_x - 2):
 			cur_map.field[y+1][x+1].onload = []
@@ -161,6 +163,8 @@ def set_bkg(bkg):
 
 #what will appear around edges of map (must be walk 0 to keep hero penned in)
 def set_border(edge):
+	global border
+	border = edge
 	for y in range(mapsize_y):
 		cur_map.field[y][0].onload = []
 		cur_map.field[y][0].onload.append("pix " + edge)
@@ -229,7 +233,7 @@ def set_tile(x, y, name):
 
 	#if the tile is an item, add it to the current tile
 	tile_prefix = name.split("/", 1)[0]
-	if tile_prefix == "items":
+	if tile_prefix == "items" and cur_mode != "remove":
 		add_item(x, y, name)
 	elif cur_mode == "add":
 		cur_map.field[y][x].onload.append("addpix " + name);
@@ -603,21 +607,31 @@ def edit_monsters(i):
 		monster_textbox.insert(END, monster_name + "\n")
 	monster_textbox.grid(row=0, column=0, columnspan=2)
 
+	monster_background_textbox = Entry(monster_window)
+	monster_background_textbox.grid(row=1, column=0, columnspan=2)
+	monster_background_textbox.delete(0, END)
+	monster_background_textbox.insert(END, cur_map.battle_background_name)
+
 	ok_button = Button(monster_window, text="OK",
-		command=(lambda: change_monsters(monster_textbox.get(1.0, END))))
+		command=(lambda: change_monsters(monster_textbox.get(1.0, END),
+					monster_background_textbox.get())))
 	ok_button.grid(row=2, column=0)
 	cancel_button = Button(monster_window, text="Cancel",
 		command=(lambda: close_monster_window()))
 	cancel_button.grid(row=2, column=1)
 	monster_window.master.wait_window(monster_window)
 
-#given a \n separated list of monsters, place into the monster list
-def change_monsters(monster_text):
+#given a \n separated list of monsters, place into the monster list.
+#Also do the same with the battle background
+def change_monsters(monster_text, monster_background):
 	cur_map.monster = []
 	monster_array = monster_text.split("\n")
 	for monster_name in monster_array:
 		if monster_name.strip() != "":
 			cur_map.monster.append(monster_name)
+
+	if monster_background != "":
+		cur_map.battle_background_name = monster_background
 	close_monster_window()
 
 def close_monster_window():
