@@ -58,7 +58,8 @@ class map:
 		self.field = [] #multidimentional array of tile class members.
 		self.monster = [] #monsters that can attack on this level.
 
-		#filename of the background image used in battles. Used in the map editor
+		#filename of the background image used in battles.
+		#Used in the map editor, to allow resaving background.
 		self.battle_background_name = ""
 		#Filled in with a Tkinter image reference
 		self.battle_background = ""
@@ -69,159 +70,97 @@ class map:
 
 		#grab the needed data from the file
 		if mapname == "": return
-		map_loc = g.mod_directory + "/data/maps/" + mapname
-		try: map_file = open(map_loc, 'r')
-		except IOError: return
+		map_file = g.read_script_file("/data/maps/" + mapname)
 
-		#go through all lines of the file up to ":def"
-		map_line = map_file.readline()
-		while map_line != '' and map_line[:1] != ":":
-			#strip out leading and trailing whitespace
-			map_line = map_line.strip()
-
-			#ignore blank lines and comments
-			if map_line[:1] == "#" or map_line[:1] == "" \
-											or map_line[:1] == "\n":
-				map_line = map_file.readline()
-				continue
-
-			#add the line
-			line = map_line.split(" ")
-			self.maptext.append(line)
-
-			#ready another line
-			map_line = map_file.readline()
-
-		#input the map data
-		map_line = map_file.readline()
-		while map_line != '' and map_line[:1] != ":":
-			#strip out spaces/tabs
-			map_line = map_line.strip()
-
-			#ignore blank lines and comments
-			if map_line[:1] == "#" or map_line[:1] == "":
-				map_line = map_file.readline()
-				continue
-
-			#allow the \ line-continuation character.
-			if map_line[-1:] == "\\": #really a single backslash, BTW.
-				map_line = map_line[:-1] + map_file.readline()
-
-			map_command = map_line.split("=", 2)[0]
-			map_command = map_command.strip()
-			if map_command.lower() == "monster":
-				self.monster.append(map_line.split("=", 1)[1])
-			elif map_command.lower() == "battle_bg":
-				self.battle_background = \
-					g.backgrounds[map_line.split("=", 1)[1].strip()]
-				self.battle_background_name = \
-					map_line.split("=", 1)[1].strip()
-			else:
-				print "bad command of " + map_command + " received"
-
-			#ready another line
-			map_line = map_file.readline()
-
-		if self.battle_background == "":
-			self.battle_background = g.backgrounds["generic.png"]
-		#input tile data
+		#current mode. 0 for the actual map, 1 for monsters,
+		#2 for tile defines, 3 for per-tile info
+		curr_mode = 0
 		current_tile = ""
-		mode=0
-		while map_line != '':
-			#strip out spaces/tabs
-			map_line = map_line.strip()
+		#0 for onload, 1 for action.
+		tilemode=0
+		for map_line in map_file:
+			#go through all lines of the file up to ":def"
+			if curr_mode == 0:
+				if map_line == '' or map_line[:1] == ":":
+					curr_mode = 1
+				else:
+					#add the line
+					line = map_line.split(" ")
+					self.maptext.append(line)
 
-			#ignore blank lines and comments
-			if map_line[:1] == "#" or map_line[:1] == "":
-				map_line = map_file.readline()
+			#input monster data
+			elif curr_mode == 1:
+				if map_line == '' or map_line[:1] == ":":
+					curr_mode = 2
+				else:
+					map_command = map_line.split("=", 2)[0].strip().lower()
+					if map_command == "monster":
+						self.monster.append(map_line.split("=", 1)[1])
+					elif map_command == "battle_bg":
+						self.battle_background = \
+							g.backgrounds[map_line.split("=", 1)[1].strip()]
+						self.battle_background_name = \
+							map_line.split("=", 1)[1].strip()
+					else:
+						print "bad command of " + map_command + " received"
+
+			#input tile data
+			if curr_mode == 2:
+				if map_line == '':
+					break
+				#if per-tile information was given
+				if map_line[:2].lower() == "xy":
+					curr_mode = 3
+					continue
+
+				#is this line giving us a new tile?
+				if map_line[:1] == ":":
+					current_tile = map_line[1:]
+					self.tiles[current_tile] = [[], []]
+					tilemode=0
+					continue
+
+				map_command = map_line.strip()
+				if map_command.lower() == "onload":
+					tilemode=0
+					continue
+				if map_command.lower() == "action":
+					tilemode=1
+					continue
+
+				if tilemode == 0: #onload
+					self.tiles[current_tile][0].append(map_command)
+				else: #action
+					self.tiles[current_tile][1].append(map_command)
+
+			if curr_mode == 3:
+				if map_line == '':
+					break
+				if map_line.lower() == "tiles": #switch back
+					curr_mode = 2
+					continue
+				#is this line giving us a new tile?
+				if map_line[:1] == ":":
+					curr_x = int(map_line[1:].split()[0].strip())
+					curr_y = int(map_line[1:].split()[1].strip())
+					self.per_tile_info.append([curr_x, curr_y, [], []])
+					tilemode=0
+					continue
+
+				map_command = map_line.strip()
+				if map_command.lower() == "onload":
+					tilemode=0
+					continue
+				if map_command.lower() == "action":
+					tilemode=1
+					continue
+
+				if tilemode == 0: #onload
+					self.per_tile_info[len(self.per_tile_info)-1][2].append(map_command)
+				else: #action
+					self.per_tile_info[len(self.per_tile_info)-1][3].append(map_command)
 				continue
 
-			#allow the \ line-continuation character.
-			while map_line[-1:] == "\\":  #really a single backslash, BTW.
-				map_line = map_line[:-1] + map_file.readline()[:-1] #for \n
-			#is this line giving us a new tile?
-			if map_line[:1] == ":":
-				current_tile = map_line[1:]
-#				self.add_tile(current_tile)
-				map_line = map_file.readline()
-				self.tiles[current_tile] = [[], []]
-				mode=0
-				continue
-
-			#if per-tile information was given
-			if map_line[:2].lower() == "xy":
-				map_line = map_file.readline()
-				map_line = map_line.strip()
-
-				while map_line != '':
-					#strip out spaces/tabs
-					map_line = map_line.strip()
-
-					#ignore blank lines and comments
-					if map_line[:1] == "#" or map_line[:1] == "":
-						map_line = map_file.readline()
-						continue
-
-					#allow the \ line-continuation character.
-					while map_line[-1:] == "\\":  #really a single backslash.
-						map_line = map_line[:-1] + map_file.readline()[:-1] #for \n
-
-
-					if map_line.lower() == "tiles": #switch back
-						map_line = map_file.readline()
-						break
-
-					#is this line giving us a new tile?
-					if map_line[:1] == ":":
-						curr_x = int(map_line[1:].split()[0].strip())
-						curr_y = int(map_line[1:].split()[1].strip())
-						self.per_tile_info.append([curr_x, curr_y, [], []])
-						map_line = map_file.readline()
-						tilemode=0
-						continue
-
-					map_command = map_line.strip()
-					if map_command.lower() == "onload":
-						tilemode=0
-						map_line = map_file.readline()
-						continue
-
-					if map_command.lower() == "action":
-						tilemode=1
-						map_line = map_file.readline()
-						continue
-
-					elif tilemode == 0: #onload
-						self.per_tile_info[len(self.per_tile_info)-1][2].append(map_command)
-					else: #action
-						self.per_tile_info[len(self.per_tile_info)-1][3].append(map_command)
-					map_line = map_file.readline()
-				continue
-
-			map_command = map_line.strip()
-			if map_command.lower() == "onload":
-				mode=0
-				map_line = map_file.readline()
-				continue
-
-			if map_command.lower() == "action":
-				mode=1
-				map_line = map_file.readline()
-				continue
-
-
-
-
-			elif mode == 0: #onload
-				self.tiles[current_tile][0].append(map_command)
-			else: #action
-				self.tiles[current_tile][1].append(map_command)
-
-			#ready another line
-			map_line = map_file.readline()
-
-		map_file.close()
-
 		#place the formatted data into self.field
 		cur_line=0
 		for line in self.maptext:
@@ -246,6 +185,8 @@ class map:
 				self.field[line[1]][line[0]].onload.append(onload_line)
 			for action_line in line[3]:
 				self.field[line[1]][line[0]].actions.append(action_line)
+		if self.battle_background == "":
+			self.battle_background = g.backgrounds["generic.png"]
 
 	def add_tile(self, tile_name):
 		new_tile = tile(tile_name)
@@ -283,52 +224,21 @@ def read_maps():
 		maps.append(map(mapname))
 
 
-
-#given a filename, return the script contained in the file.
-#called from read_scripts()
-def read_script_file(file_name):
-	temp_array = []
-	file = open(file_name, 'r')
-
-	#go through all lines of the file
-	file_line = file.readline()
-	while file_line != '':
-		#strip out spaces/tabs
-		file_line = file_line.strip()
-
-		#ignore blank lines and comments
-		if file_line[:1] == "#" or file_line[:1] == "":
-			file_line = file.readline()
-			continue
-
-		#allow the \ line-continuation character.
-		while file_line[-1:] == "\\":  #really a single backslash, BTW.
-			file_line = file_line[:-1] + file.readline()[:-1]
-
-		temp_array.append(file_line)
-
-		#ready another line
-		file_line = file.readline()
-
-	file.close()
-	return temp_array
-
-
 endgame_act = []
 wingame_act = []
 newgame_act = []
 levelup_act = []
 #read scripts from disk, and place in *_act[]
 def read_scripts():
-	#This is a bit weird, but Python does not allow the simple way.
+	#This is a bit weird, but Python does not seem to allow the simple way.
 	global endgame_act
 	global wingame_act
 	global newgame_act
 	global levelup_act
-	endgame = read_script_file(g.mod_directory + "/data/endgame.txt")
-	wingame = read_script_file(g.mod_directory + "/data/wingame.txt")
-	newgame = read_script_file(g.mod_directory + "/data/newgame.txt")
-	levelup = read_script_file(g.mod_directory + "/data/levelup.txt")
+	endgame = g.read_script_file("/data/endgame.txt")
+	wingame = g.read_script_file("/data/wingame.txt")
+	newgame = g.read_script_file("/data/newgame.txt")
+	levelup = g.read_script_file("/data/levelup.txt")
 
 	for line in endgame: endgame_act.append(line)
 	for line in wingame: wingame_act.append(line)
