@@ -162,6 +162,11 @@ def sell_item():
 	if curr_focus != 0: return 0
 	if curr_item > len(item.inv): return 0
 	if item.inv[curr_item] == -1: return 0
+
+	if item.item[item.inv[curr_item]].price == 0:
+		main.print_message("You feel attached to your " +
+			item.item[item.inv[curr_item]].name)
+		return 0
 	#give the player money
 	#note that all stores have a 5 year, money-back guarantee,
 	#and accept returns from other stores as well.
@@ -370,6 +375,8 @@ def init_window_shop(store_type_input):
 	curr_item = 0
 	global curr_focus
 	curr_focus = 1
+	global cur_button
+	cur_button = 2
 
 	global store_num
 	for i in range(len(g.shops)):
